import { serializeError } from 'serialize-error';
import { checkPermission } from './checkPermissions.js';
import { extractUrlParts, isRequestBodyValid, productionErrorMessage, } from './utils.js';
export function createClient(webMethods) {
    return {
        async parseRequest(request) {
            const url = new URL(request.url);
            const parts = extractUrlParts(url.pathname);
            if (!parts) {
                throw new Error('Invalid request URL');
            }
            return parts;
        },
        async processRequest(request, devMode = false) {
            const parts = extractUrlParts(request.url);
            if (!parts) {
                return {
                    response: new Response('invalid request', { status: 400 }),
                    hasError: false,
                };
            }
            const body = (await request.json());
            if (!isRequestBodyValid(body)) {
                return {
                    response: new Response('invalid request', { status: 400 }),
                    hasError: false,
                };
            }
            const loadWebMethodFile = webMethods[`/${parts.filename}`];
            try {
                if (!loadWebMethodFile) {
                    throw new Error(`Error loading web module ${parts.filename}: Cannot find module '${parts.filename}'`);
                }
                const webMethod = (await loadWebMethodFile())[parts.methodname];
                if (!webMethod) {
                    throw new Error(`Error loading function from web module ${parts.filename}: function '${parts.methodname}' not found`);
                }
                await checkPermission(request, webMethod.permission);
                return {
                    response: Response.json({
                        result: await webMethod(...body),
                    }),
                    hasError: false,
                };
            }
            catch (error) {
                const serializedError = serializeError(error, { maxDepth: 1 });
                return {
                    response: Response.json({
                        result: devMode || !(error instanceof Error)
                            ? serializedError
                            : {
                                ...serializedError,
                                message: productionErrorMessage,
                                stack: productionErrorMessage,
                            },
                        exception: true,
                    }),
                    error,
                    hasError: true,
                };
            }
        },
    };
}
